package com.tridion.deployer.modules;

import java.io.File;
import java.io.IOException;
import java.io.StringReader;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.commons.lang3.StringEscapeUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.tridion.broker.taxonomies.TaxonomyCMTransformer;
import com.tridion.configuration.Configuration;
import com.tridion.configuration.ConfigurationException;
import com.tridion.deployer.Module;
import com.tridion.deployer.ProcessingException;
import com.tridion.deployer.Processor;
import com.tridion.storage.si4t.FactoryAction;
import com.tridion.storage.si4t.IndexType;
import com.tridion.storage.si4t.SearchIndexProcessor;
import com.tridion.storage.si4t.TridionPublishableItemProcessor;
import com.tridion.taxonomies.Keyword;
import com.tridion.transport.transportpackage.GenericTransportItem;
import com.tridion.transport.transportpackage.SchemaKey;
import com.tridion.transport.transportpackage.Section;
import com.tridion.transport.transportpackage.TaxonomyKey;
import com.tridion.transport.transportpackage.TransportPackage;
import com.tridion.util.TCMURI;

@SuppressWarnings("deprecation")
public class SearchExtensionTaxonomyDeploy extends Module {
	SearchIndexProcessor _processor;
	TransportPackage currentPackage;
	String _transactionId;
	private String storageId = "";
	private static final DocumentBuilderFactory DBF = DocumentBuilderFactory.newInstance();
	private Logger log = LoggerFactory.getLogger(SearchExtensionTaxonomyDeploy.class);

	public SearchExtensionTaxonomyDeploy(Configuration config, Processor processor)
			throws ConfigurationException {
		super(config, processor);
	}

	protected boolean setup() {
		_transactionId = currentPackage.getTransactionHeader().getTransactionId().toString();
		SearchModuleUtility util = new SearchModuleUtility(this);
		_processor = util.initalizeSearchProcessor(currentPackage);

		return _processor!= null;
	}


	protected void processKeyword(Keyword keyword, TCMURI taxonomyURI, String taxonomyName) throws ProcessingException
	{
		String keywordSource = getKeywordSource(keyword, taxonomyURI, taxonomyName);
		String uniqueTransactionId = _transactionId;
		TridionPublishableItemProcessor tp = new TridionPublishableItemProcessor(
				keywordSource, FactoryAction.UPDATE, IndexType.TAXONOMY,
				Integer.toString(taxonomyURI.getPublicationId()), 
				keyword.getKeywordURI(), 
				uniqueTransactionId, 
				storageId);
		try {
			tp.processKeywordSource();
		} catch (Exception ex){
			throw new ProcessingException("Could not deploy keyword to search indexer " + keyword, ex);
		}finally{}
	}

	@Override
	public void process(TransportPackage data) throws ProcessingException {
		currentPackage = data;
		Section taxonomySection = data.getProcessorInstructions().getSection("Taxonomies");
		if ((setup()) && (taxonomySection != null))
		{
			Iterator iterator = taxonomySection.getFileItems();
			log.debug("Processing section " + taxonomySection.getName());
			while (iterator.hasNext())
			{
				Object item = iterator.next();
				processItem(item, taxonomySection);
			}
		}
	}

	protected void processItem(Object item, Section taxonomySection)
			throws ProcessingException
	{
		try {
			if ((item instanceof TaxonomyKey))
			{
				TaxonomyKey transportItem = (TaxonomyKey)item;

				TCMURI itemURI = transportItem.getId();
				String itemName = transportItem.getName();

				boolean isInUseForIdentification = Boolean.valueOf(transportItem.isUseForIdentification()).booleanValue();

				File taxonomyFile = new File(currentPackage.getLocationPath() + taxonomySection.getRelativePath() + itemName);

				Map schemaTypeMaps = new HashMap();
				ItemDeploySchemaManager itemDeploySchemaManager = new ItemDeploySchemaManager(currentPackage);
				if (transportItem.getSchemas() != null) {
					for (SchemaKey schema : transportItem.getSchemas())
					{
						TCMURI schemaURI = schema.getId();
						if ((schemaURI != null) && (!schemaURI.equals("tcm:0-0-0")))
						{
							String simulateDom = "<Schema Id=\"" + schemaURI.toString() + "\"/>";
							try
							{
								Document schemaDoc = DBF.newDocumentBuilder().parse(new InputSource(new StringReader(simulateDom)));
								GenericTransportItem tpItem = new GenericTransportItem(schemaDoc);
								log.debug("Schema present for Taxonomy, loading schema information: " + schemaURI);
								itemDeploySchemaManager.setSchemaInformationOnItem(tpItem, schemaURI);

								schemaTypeMaps.put(tpItem.getSchemaNamespace(), tpItem.getTypeMap());
							}
							catch (ParserConfigurationException pce)
							{
								log.warn("Got configuration exception while trying to build other transport items: " + pce.getMessage());
							}
							catch (IOException ioe)
							{
								log.warn("Got IO exception while trying to build other transport items: " + ioe.getMessage());
							}
							catch (SAXException saxe)
							{
								log.warn("Got SAX exception while trying to build other transport items: " + saxe.getMessage());
							}
						}
					}
				}
				TaxonomyCMTransformer taxonomyTransformer = new TaxonomyCMTransformer(schemaTypeMaps, isInUseForIdentification);
				Collection<Keyword> taxonomyMetaFacets = taxonomyTransformer.transformTaxonomy(taxonomyFile);

				Iterator taxIterator = taxonomyMetaFacets.iterator();
				String taxonomyName = "";
				int i = 0;
				while (taxIterator.hasNext())
				{

					Object oKeyword = taxIterator.next();
					if(oKeyword instanceof Keyword)
					{
						Keyword keyword = (Keyword) oKeyword;
						if(i++==0)	
						{
							taxonomyName = keyword.getKeywordName();
						}
						else
						{
							processKeyword(keyword, itemURI, taxonomyName);
						}
					}
				}

				_processor.triggerIndexing(_transactionId);
			}
		} catch (Exception ex) {
			throw new ProcessingException("Could not deploy taxonomy to search indexer " + taxonomySection, ex);
		} 
		finally
		{
			SearchIndexProcessor.debugLogRegister(log);
			SearchIndexProcessor.cleanupRegister(_transactionId, log);
		}
	}

	private String getKeywordSource(Keyword keyword, TCMURI taxonomyURI, String taxonomyName)
	{
		String keywordURI = keyword.getKeywordURI();
		String keywordParent = keyword.getParentKeyword().getKeywordURI();		
		String TopKeywordParent = getKeywordFirstLevel(keyword, taxonomyURI);
		String keywordName = StringEscapeUtils.escapeXml(keyword.getKeywordName());
		int keywordType = taxonomyURI.getItemId();
		String esTaxonomyName = StringEscapeUtils.escapeXml(taxonomyName);
		//Add publication ID to the search index
		Integer publicationid = taxonomyURI.getPublicationId();

		String keywordIndexDataFormat = "<Keyword><!-- INDEX-DATA-START:<indexdata><url>%1$s</url><title>%2$s</title><type>%3$s</type><typename>%4$s</typename><itemtype>1024</itemtype><publicationid>%5$s</publicationid><keywordParent>%6$s</keywordParent><TopKeywordParent>%7$s</TopKeywordParent></indexdata>:INDEX-DATA-END --></Keyword>";
		return String.format(keywordIndexDataFormat, keywordURI, keywordName, keywordType, esTaxonomyName, publicationid, keywordParent, TopKeywordParent);
	}
	private String getKeywordFirstLevel(Keyword keyword, TCMURI taxonomyURI)
	{

		Keyword parentKeyword = keyword;
		String taxID = taxonomyURI.toString();

		while (parentKeyword.getParentKeyword().getKeywordURI().equals(taxID) == false){
			parentKeyword = parentKeyword.getParentKeyword();
		}

		return parentKeyword.getKeywordURI();
	}
}
