package com.tridion.deployer.modules;

import java.util.Iterator;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tridion.broker.HomeFactory;
import com.tridion.broker.taxonomies.TaxonomyHome;
import com.tridion.configuration.Configuration;
import com.tridion.configuration.ConfigurationException;
import com.tridion.deployer.Module;
import com.tridion.deployer.ProcessingException;
import com.tridion.deployer.Processor;
import com.tridion.storage.si4t.IndexType;
import com.tridion.storage.si4t.SearchIndexProcessor;
import com.tridion.storage.si4t.TridionBaseItemProcessor;
import com.tridion.taxonomies.Keyword;
import com.tridion.transport.transportpackage.ProcessorInstructions;
import com.tridion.transport.transportpackage.TaxonomyKey;
import com.tridion.transport.transportpackage.TransportPackage;
import com.tridion.util.TCDURI;
import com.tridion.util.TCMURI;

@SuppressWarnings("deprecation")
public class SearchExtensionTaxonomyUndeploy extends Module {

	SearchIndexProcessor _processor;
	TransportPackage currentPackage;
	String _transactionId;
	private String storageId = "";

	private Logger log = LoggerFactory.getLogger(SearchExtensionTaxonomyUndeploy.class);

	public SearchExtensionTaxonomyUndeploy(Configuration config, Processor processor)
			throws ConfigurationException {
		super(config, processor);
	}

	protected boolean setup(TransportPackage currentPackage) {
		_transactionId = currentPackage.getTransactionHeader().getTransactionId().toString();
		SearchModuleUtility util = new SearchModuleUtility(this);
		_processor = util.initalizeSearchProcessor(currentPackage);

		return _processor!= null;
	}

	protected void processKeyword(Keyword keyword, TCMURI taxonomyURI, String taxonomyName)
	{
		TridionBaseItemProcessor.registerItemRemoval(
				_transactionId, 
				keyword.getKeywordURI(), 
				IndexType.TAXONOMY, 
				log, 
				Integer.toString(taxonomyURI.getPublicationId()), 
				this.storageId);
	}


	protected void processItem(Object item)
			throws ProcessingException
	{
		try {
			if ((item instanceof TaxonomyKey))
			{
				TaxonomyKey transportItem = (TaxonomyKey)item;

				TCMURI itemURI = transportItem.getId();
				String itemName = transportItem.getName();

				TCDURI taxonomyURI = new TCDURI(itemURI.getPublicationId(), 281474976710720L, itemURI.getItemId());

				TaxonomyHome taxonomyHome = (TaxonomyHome)HomeFactory.findHome(taxonomyURI);
				List taxonomyMetaFacets = taxonomyHome.findTaxonomy(itemURI.getPublicationId(), (int)itemURI.getItemId());
				if ((taxonomyMetaFacets == null) || (taxonomyMetaFacets.isEmpty())) {
					log.debug("Could not undeploy Taxonomy " + itemURI + " because it was not deployed");
					return;
				}
				Iterator taxIterator = taxonomyMetaFacets.iterator();
				String taxonomyName = "";
				int i = 0;
				while (taxIterator.hasNext())
				{
					Object oKeyword = taxIterator.next();
					if(oKeyword instanceof Keyword)
					{
						Keyword keyword = (Keyword) oKeyword;
						if(i++==0)	
						{
							taxonomyName = keyword.getKeywordName();
						}
						else
						{
							processKeyword(keyword, itemURI, taxonomyName);
						}
					}
				}

				_processor.triggerIndexing(_transactionId);
			}

		} catch (Exception ex) {
			throw new ProcessingException("Could not undeploy taxonomy from search indexer " + item, ex);
		} 
		finally
		{
			SearchIndexProcessor.debugLogRegister(log);
			SearchIndexProcessor.cleanupRegister(_transactionId, log);
		}
	}

	@Override
	public void process(TransportPackage data) throws ProcessingException {
		currentPackage = data;

		if (setup(data))
		{
			ProcessorInstructions instructions = data.getProcessorInstructions();
			Iterator iterator = instructions.getArguments();
			while (iterator.hasNext())
			{
				Object item = iterator.next();
				processItem(item);
			}
		}
	}
}
